define(['jquery','backbone','underscore','tools','editor/pane','editor/actions'],function($,$$,_,tools,Pane,Actions){
	var gcount=0;
	return $$.Model.extend({
		defaults:{
			'css':'libs/bootstrap/css/bootstrap.min.css,libs/bootstrap-glyphicons-master/css/bootstrap.icon-large.min.css'
			,'less':'assets/editor/less/editor.less'
			,"wizardsDirectory":"editor/wizards"
			,'layout':{
				'default':'assets/editor/layouts/default.json'
			}
			,"selectedObject": null
			,"gameObject":null
			,"isSubGui":false
		}
		,wizards:null
		,panes:null
		,currentLayout:null
		,dom:null
		,actions:null
		,initialize:function(attributes,options){
			var self=this;
			if(!this.get("isSubGui")){
				this.InjectCSS();
				this.InjectLess(function(){self.StylesheetsReady();});
			}else{
				this.StylesheetsReady();
			}

			this.on("change:selectedObject",function(){
				var last=this._previousAttributes.selectedObject, o = this.get("selectedObject");
				
				if(last && last.set){
					last.set("drawOutline",false);
				}

				if(o && o.set){
					o.set("drawOutline",true);
				}
			});

			this.actions = new Actions({gui:this});
		}
		,StylesheetsReady:function(){
			this.set("id",gcount++);
			this.dom = $('<div />').addClass("editor").attr("id","editor_"+gcount).appendTo(document.body);
			this.InitPanes();
			this.InitLayout();
		}
		,InjectCSS:function(){
			var css = this.get("css");
			if(!css)
				return;
			
			css = css.split(',');
			_.each(css, function(file){
				$("head").append($('<link />').attr({href:file,type:'text/css',rel:'stylesheet'}));
			});
		}
		,InjectLess:function(callback){
			var less = this.get("less");
			if(!less)
				return;

			less = less.split(',');
			_.each(less, function(file){
				$("head").append($('<link />').attr({href:file,type:'text/css',rel:'stylesheet/less'}));
			});

			require(['less'],function(less){
				//less.watch();
				callback&&callback();
			});
		}
		,GetWizard:function(Obj,callback,parentObject,role){
			var self=this, className, wizardClassName, wizard;
			
			if(_.isString(Obj)){
				className = Obj;
				Obj = null;
			}else if(!Obj.meta || !Obj.meta.className){
				callback&&callback(null);
				return;
			}else{
				className = Obj.meta.className;
			}

			if(!this.wizards)
				this.wizards = {};
			
			wizardClassName = this.get("wizardsDirectory") + "/" + className;

			if(this.wizards[wizardClassName]){
				callback&&callback(new (this.wizards[wizardClassName])({targetObject:Obj,gui:this,parentObject:parentObject,role:role}));
				return;
			}

			try{
				require([wizardClassName],function(wClass){
					self.wizards[wizardClassName] = wClass;
					callback&&callback(new wClass({targetObject:Obj,gui:self,parentObject:parentObject,role:role}));
				});	
			}catch(e){
				callback&&callback(null);
			}
			
		}
		,ObjectAttributeGetWizard:function(Obj,attribute, callback){
			if(!Obj.meta || !Obj.meta.attributes || !Obj.meta.attributes[attribute]){
				callback&&callback(null);
				return;
			}

			this.GetWizard(Obj.meta.attributes[attribute], callback,Obj,attribute);
		}
		,InitPanes:function(){
			this.panes = new $$.Collection;
			
			var self=this, resize;
			$(window).on("resize",resize = function(){
				self.arrangePanes();
			});

			this.panes.on("add",function(pane){
				pane.GetContainer().appendTo(self.dom);
				resize();
			});
			this.on("resize",resize);
		}
		,arrangePanes:function(){
			var self=this, w = self.dom.innerWidth(), h=self.dom.innerHeight(),freespaceX=w,freespaceY=h,l=self.currentLayout;
			if(!l){
				return;
			}

			self.panes.each(function(pane){
				switch(pane.get("positioning")){
					case 'dockvertical':
						freespaceY -= pane.get("height");
					break;
				}
			});

			self.panes.each(function(pane){
				var container = pane.GetContainer();
				
				switch(pane.get("positioning")){
					case 'dockleft':
						container.css({
							"float":"left"
							,"width":pane.get("width") || "auto"
							,"height":freespaceY
						});
					break;
					case 'dockright':
						container.css({
							"float":"right"
							,"width":pane.get("width") || "auto"
							,"height":freespaceY
						});
					break;
					case 'dockvertical':
						container.css({
							"float":"none"
							,"height":pane.get("height")
							,"width":"100%"
						});
					break;
					case "auto":
					default:
						container.css({
							"float":"none"
							,"width":"auto"
							,"height":freespaceY
						});
				}

				container.css("overflow",pane.get("scrollable")?"auto":"hidden");
				pane.trigger("resize");
			});
		}
		,InitLayout:function(){
			var l=this.get("layout");
			if(!l.default){
				return;
			}

			this.SetLayout(l.default);
		}
		,SetLayout:function(l){
			var self=this;
			if(_.isString(l)){
				l = tools.FileGetJSON(l);
			}
			this.currentLayout = l;
			_.each(l.panes,function(v,k){
				v.name = k;
				v.parent = self;
				if(!self.panes.some(function(pane){return pane.get("name")==k;})){
					self.panes.add(new Pane(v));	
				}else{
					self.panes.find(function(pane){return pane.get("name")==k;}).set(v);
				}
			});
			this.trigger("resize");
		}
		,Show:function(){
			this.dom.show();
		}
		,Hide:function(){
			this.dom.hide();
		}
		,Destroy:function(){
			this.panes.each(function(pane){
				pane.GetContainer().remove();
			});
			this.panes.reset();
		}
	});
});