define(['editor/widget','jquery','tools','underscore','entities/container','entities/entity','scenes/scene', 'game', 'entities/module'],function(Widget,$,tools,_,Container,Entity,Scene, Game, Module){
	return Widget.extend({
		defaults:_.extend({},Widget.prototype.defaults,{
			'title': 'Project Tree'
			,'dataObject': null
			,'collapsed': false
			,'collapsable': false
			,'onlySelectedObject': false
			,'displayModules': true
			,'depth':0
		})
		,initialize:function(attributes,options){
			Widget.prototype.initialize.apply(this,arguments);
			var self = this, editor = this.get("parent").get("parent"), selectedObject;
			if(this.get("onlySelectedObject")){
				selectedObject = editor.get("selectedObject");
				editor.on("change:selectedObject",function(){
					self.set("dataObject",editor.get("selectedObject"));
					self.UpdateTree();
				});
			}else{
				selectedObject = editor.get("gameObject");
				editor.on("gameObjectChangedScene",function(){
					self.set("dataObject",editor.get("gameObject"));
					self.UpdateTree();
				});
				editor.on("change:selectedObject",function(){
					self.UpdateTree();
				});
			}
			
			
			if(selectedObject){
				this.set("dataObject",selectedObject);
			}

			if(this.get("dataObject")){
				this.UpdateTree();
			}

			this.on("action:select",this.OnSelect);
		}
		,widgetBodyTemplate:""
		,UpdateTree:function(){
			var self = this, i=0, subTree, body = this.GetWidgetBody(), sO = this.get("parent").get("parent").get("selectedObject"), displayModules=this.get("displayModules"), depth=this.get("depth")||0;
			body.empty();

			if(!this.get("dataObject"))
				return;

			this.refs = [];

			$('<div class="tree" />').append($('<ul />').html((subTree = function(data,level){
				var icon,instof,mods,hasChildren, html = "", active=false;
				level = level||0;

				if(depth && level>=depth){
					return html;
				}

				if(data instanceof Game){
					data = data.get("scene");
				}

				if(!data || (!(data instanceof Container) && !(data instanceof Module))){
					return html;
				}

				active = data===sO;

				hasChildren = !!data.__childEntities;

				icon = '<i class="icon-file '+(active?"icon-white":"")+'"></i>';

				if(data instanceof Scene){
					icon = '<i class="icon-globe '+(active?"icon-white":"")+'" data-toggle="tooltip" title="Scene" ></i>';
				}

				if(data instanceof Entity){
					icon = '<i class="icon-map-marker '+(active?"icon-white":"")+'" data-toggle="tooltip" title="Entity" ></i>';
					if(displayModules){
						mods = data.modules;
					}						
					if(!data.ready){
						data.on("ready",function(){self.UpdateTree();});
					}
				}

				if(data instanceof Module){
					icon = '<i class="icon-leaf '+(active?"icon-white":"")+'" data-toggle="tooltip" title="Module" ></i>';
				}

				html += "<li>";

				//html += "<a href='#' data-tree-id='"+i+"' data-action='select' >"+data.get("name")+"</a>";

				html += '<input type="checkbox" id="tree-item-'+i+'" '+(!hasChildren&&!mods?'disabled="disabled"':'')+' /><label for="tree-item-'+i+'" '+(active?"class='active'":"")+'><a href="#" data-tree-id="'+i+'" data-action="select">'+icon+' '+data.get("name")+'</a></label>';

				self.refs[i] = data;
				++i;

				if(!hasChildren && !mods){
					return html + "</li>";
				}

				html += '<ul>';

				if(mods){
					_.each(mods,function(ent){
						html += subTree(ent,level+1);
					});
				}

				_.each(data.__childEntities,function(ent){
					html += subTree(ent,level+1);
				});

				return html + "</ul></li>";
			})(this.get("dataObject")))).appendTo(body);

			body.find('input').attr("checked","checked");
			body.find('[data-toggle="tooltip"]').tooltip();
		}
		,OnSelect:function(element){
			var editor = this.get("parent").get("parent"), target = this.refs[parseInt(element.data("tree-id"))];
			
			this.GetWidgetBody().find(".active").removeClass("active");
			this.GetWidgetBody().find(".icon-white").removeClass("icon-white");

			if(target !== editor.get("selectedObject")){
				/*element.parent(":first").addClass("active");
				element.find('i').addClass('icon-white');*/
				editor.set("selectedObject", target);
			}else{
				editor.set("selectedObject", null);
			}
		}
	});
});