define(['math/vec2', 'backbone', 'underscore','tools','entities/container'],function(vec2,$$,_,tools,Container){
	var entityCount = 0, Entity;
	return (Entity = Container.extend({
		meta:{
			className:"entities/entity"
			,attributes:{
				position:"math/vec2"
				,name:'String'
				,rotation:'Number'
				,scale:"math/vec2"
				,origin:"math/vec2"
				,modules:"entities/module"
				,entities:"entities/entity"
			}
		}
		,defaults:{
			position:null
			,name:"unknown-entity"
			,id:0
			,scale:null
			,origin:null
			,rotation:0
			,modules:null
			,entities:null
			,parent:null
			,game:null
		}
		,texture:null
		,modules:null
		,ready:false
		,visible:true
		,initialize:function(attributes, options){
			this.attributes.id = ++entityCount;
			this.attributes.position = attributes&&attributes.position ? (new vec2(attributes.position.x,attributes.position.y)) : (new vec2);
			this.attributes.scale = attributes&&attributes.scale? new vec2(attributes.scale.x,attributes.scale.y) : new vec2(1,1);
			this.attributes.origin = attributes&&attributes.origin? new vec2(attributes.origin.x,attributes.origin.y) : new vec2;
			
			if(this.attributes.entities){
				for(i=0;i<this.attributes.entities.length;++i){
					ent = new Entity(this.attributes.entities[i]);
					if(ent.ready){
						ent.visible = true;
					}else{
						ent.visible = false;
						ent.on("ready",function(){
							this.visible = true;
						});
					}
					this.AddEntity(ent);
				}
			}

			//delegate events
			this.on("update",function(time,delta){
				var self=this;
				_.each(this.__childEntities,function(entity){entity.trigger("update",time,delta);});
			});

			this.on("change:game",function(){
				var self=this;
				_.each(this.__childEntities,function(entity){entity.set("game",self.attributes.game);});
			});

			//init Entity Modules
			this.modules = {};

			if(this.attributes.modules){
				this.InitModules();
			}else{
				this.ready = true;
				this.trigger("ready");
			}
		}
		,InitModules:function(){
			var self=this, ready=0;
			this.on("module-ready",function(name){
				if((--ready)<1){
					self.ready = true;
					self.trigger("ready");
				}
			});
			_.each(this.attributes.modules, function(value,key){
				ready++;
				self.AddModule(key,value);
			});
		}
		,AddModule:function(id,attributes){
			if(this.modules.hasOwnProperty(id)){
				tools.Error("Module "+id+" already exists in Entity");
				return;
			}

			var self = this, moduleClass = attributes.moduleClass;
			/*delete attributes.moduleClass;*/

			if(!_.isString(moduleClass)){
				tools.Warn('moduleClass parameter is mandatory for entity modules, configuration ignored for module: '+id);
				return;
			}

			attributes.entity = this;
			attributes.name = id;
			require([moduleClass],function(module){
				self.modules[id] = new module(attributes);
				if(!self.modules[id].ready){
					self.modules[id].on("ready",function(){
						self.trigger("module-ready",this.name);
					});
				}else{
					self.trigger("module-ready",id);
				}
			});
		}
		,RemoveModule:function(module){
			if(!this.modules)
				return false;
			for(var mod in this.modules){
				if(this.modules.hasOwnProperty(mod) && this.modules[mod] === module){
					delete this.modules[mod];
					module.trigger("destroy");
					module.set("entity",null);
					return true;
				}
			}

			return false;
		}
		,GetModuleByName:function(name){
			return this.GetModuleByAttribute("name",name);
		}
		,GetModuleByAttribute:function(attribute, value){
			return _.find(this.modules,function(m){return m.get(attribute) == value});
		}
		,GetModuleByClass:function(classDefinition){
			return _.find(this.modules,function(m){return m instanceof classDefinition});	
		}
		,toJSON:function(){
			var copy = _.clone(this.attributes);
			
			if(this.modules){
				copy.modules = {};
				_.each(this.modules,function(module,name){copy.modules[name] = module.toJSON()});
			}

			if(this.__childEntities){
				copy.entities = [];
				_.each(this.__childEntities,function(ent){copy.entities.push(ent.toJSON())});
			}

			delete copy.parent;
			delete copy.game;
			delete copy.id;

			copy.position = this.get("position").toJSON();
			copy.origin = this.get("origin").toJSON();
			copy.scale = this.get("scale").toJSON();

			return copy;
		}
	}));
});
