define(['sprites/sprite','entities/module','underscore','math/vec2','entities/modules/tilemap/editor'],function(Sprite,Module,_,Vec2,Editor){
	return Module.extend({
		meta:{
			attributes:{
				"size":"math/vec2"
			}
		}
		,defaults:_.extend({},Module.prototype.defaults,{
			'name': 			'unnamed-tilemap'
			,'editorMode': 		false
			,'sheet': 			null
			,'sheetObject': 	null
			,'tiles':			null
			,'size': 			null
			,'textureSize': 	null
			,'repeatEdge': 		true
			,'Scroll':			null
			,'editable': 		false
		})
		,context:null
		,editor:null
		,initialize:function(attributes,options){
			if(!attributes)
				return;
			
			attributes.scroll?this.set("scroll",new Vec2(attributes.scroll)):this.set("scroll",new Vec2(0,0));
			attributes.size?this.set("size",new Vec2(attributes.size)):this.set("size",new Vec2(512,512));
			attributes.textureSize?this.set("textureSize",new Vec2(attributes.textureSize)):this.set("textureSize",new Vec2());

			if(attributes.sheet)
				this.InitTileMap();

			this.on("change:editable",function(){
				this.ToggleEditable();
			});
			this.ToggleEditable();

			//resize tiles array when size is changed
			this.on("change:size",function(){
				var i=0,j=0,pad,oldSize = this._previousAttributes.size, size=this.attributes.size;
				if(size.x > oldSize.x || size.x < oldSize.x){
					pad = size.x - oldSize.x;
					while(i < size.x * size.y){
						if( i > 0 && ((i+1) % (size.x)) == 0){
							if(pad>0){
								j=pad;
								//fill size padding with -1 (transparent)
								while(j>0){
									this.attributes.tiles.splice(i+1-j,0,-1);
									--j;
								}
								i+=pad;
							}else{
								//remove negative padding from tiles array
								this.attributes.tiles.splice(i+1,pad*-1);
							}
						}
						++i;
					}
				}

				if(size.y > oldSize.y){
					this.attributes.tiles = this.attributes.tiles.concat(new Array((size.y - oldSize.y)*size.x));
				}else if(size.y < oldSize.y){
					i = size.x * size.y, j = oldSize.x * oldSize.y;
					this.attributes.tiles.splice(i - 1,j - i);
				}
			});

			var keyboard = this.attributes.entity.attributes.game.GetKeyboard();
			this.on("ready",function(){
				var self=this,x=0,y=0;
				setInterval(function(){
					if(keyboard.IsDown("left-arrow")){
						x--;
					}
					if(keyboard.IsDown("right-arrow")){
						x++;
					}

					if(keyboard.IsDown("up-arrow")){
						y--;
					}
					if(keyboard.IsDown("down-arrow")){
						y++;
					}

					self.Scroll(x,y);
				},1000/40);
			});
		}
		,InitTileMap:function(){
			var onGameAttach
				,onPaused
				,onDraw
				,game
				,entity=this.attributes.entity
				,dc = this.attributes.entity.__drawChildEntities
				,self = this
				,ctx = this.context = document.createElement('canvas').getContext("2d")
				,s = new Sprite(this.get("sheet"))
			;
			
			entity.__drawChildEntities = false;

			entity.texture = ctx.canvas;
						
			ctx.canvas.width = this.get("textureSize").x;
			ctx.canvas.height = this.get("textureSize").y;
			
			this.set("sheetObject",s);

			entity.on("draw",(onDraw = function(){
				if(self.ready)
					self.Render();
			}));

			if(s.ready){
				this.Render();
				this.ready = true;
				this.trigger("ready");
			}else{
				s.on("ready",function(){
					self.Render();
					self.ready = true;
					self.trigger("ready");
				});
			}

			onPaused = function(){
				if(game && game.get("editor")){
					self.set('editable',!!game.get("paused"));
				}
			}

			onGameAttach = function(){
				if(this.editor){
					this.editor.trigger("destroy");
				}

				if(!(game = entity.get("game"))){
					return;
				}

				if(!game.get("editor")){
					this.editor = null;
					return;
				}

				this.editor = new Editor({
					tilemap:self
					,game:game
				});


				// game.on("change:paused",onPaused);
				// onPaused();
			}

			entity.on("change:game",onGameAttach);
			onGameAttach();

			this.on("destroy",function(){
				
				entity.off("change:game",onGameAttach);
				
				if(game){
					game.off("change:paused",onPaused);
				}

				this.set("editable",false);
				
				entity.off("draw",onDraw);
				
				this.get("sheetObject").trigger("destroy");
				
				this.context = null;
				
				this.ready = false;
				
				entity.texture = null;
				
				entity.__drawChildEntities = dc;
			});
		}
		,ToggleEditable:function(){
			var self=this
				,active = this.get("editable")
				,mouse = this.get("entity").get("game").GetMouse()
				,onClick
				,onMove
			;

			onClick = function(){
				if(!self.get("editable"))
					return;
				var tmptile, rawpos = this.GetPosition(), pos = self.attributes.entity.GlobalToLocal(rawpos);

				pos.x += self.attributes.scroll.x;
				pos.y += self.attributes.scroll.y;

				tmptile = Math.floor(pos.y / self.attributes.sheet.tilesize) * self.attributes.size.x + Math.floor(pos.x / self.attributes.sheet.tilesize);

				if(tmptile === self.editModeSelectedTile){
					self.attributes.tiles[tmptile] = (self.attributes.tiles[tmptile]+1) % self.attributes.sheetObject.img.width * self.attributes.sheetObject.img.height / self.attributes.sheetObject.attributes.tilesize / self.attributes.sheetObject.attributes.tilesize ;
				}else{
					self.editModeSelectedTile = tmptile;
				}
			};

			onMove = function(){
				if(!self.get("editable"))
					return;
				var rawpos = this.GetPosition(), pos = self.attributes.entity.GlobalToLocal(rawpos);

				pos.x += self.attributes.scroll.x;
				pos.y += self.attributes.scroll.y;

				self.editModeHoveredTile = Math.floor(pos.y / self.attributes.sheet.tilesize) * self.attributes.size.x + Math.floor(pos.x / self.attributes.sheet.tilesize);
			};

			if(active){
				mouse.on("click",onClick);
				mouse.on("mousemove",onMove);
			}else{
				mouse.off("click",onClick);
				mouse.off("mousemove",onMove);
			}
		}
		,Scroll:function(x,y){
			this.attributes.scroll.x = x;
			this.attributes.scroll.y = y;
		}
		,Render:function(){
			var 
				x,xx
				,y,yy
				,width = this.attributes.size.x * this.attributes.sheet.tilesize
				,height = this.attributes.size.y * this.attributes.sheet.tilesize
				,offsetX = this.attributes.scroll.x % width
				,offsetY = this.attributes.scroll.y % height
				,immidiateOffsetX 
				,immidiateOffsetY
				,s = this.attributes.sheetObject
				,size = this.attributes.size
				,ctx = this.context
				,tile = 0
				,tiles = this.attributes.tiles
				,tileSize = this.attributes.sheet.tilesize
				,child
			;
			
			ctx.clearRect(0,0,ctx.canvas.width,ctx.canvas.height);

			ctx.save();

			//infinite repeating only works with positive offsets, so we convert negative ones
			if(offsetX < 0){
				offsetX = width + offsetX;
			}
			if(offsetY < 0){
				offsetY = height + offsetY;
			}

			immidiateOffsetX = offsetX%tileSize;
			immidiateOffsetY = offsetY%tileSize;

			// draw the complete height & a padding of tileSize*2,starting at negative tileSize, to also draw the immidiateOffsetY
			for(y = -tileSize; y < ctx.canvas.height + tileSize; y+= tileSize){
				// current y position + scrolling of the map (offsetY)
				yy = (y+offsetY);
				// draw the complete width & a padding of tileSize*2,starting at negative tileSize, to also draw the immidiateOffsetsX
				for(x = -tileSize; x < ctx.canvas.width + tileSize; x+= tileSize){
					// current x position + scrolling of the map (offsetX)
					xx = (x+offsetX);
					// to get the tile index we calculate xx + yy * size.y,
					// to make the whole tilemap repeat, we always use the modulo of the size (size.x for xx and size.y for yy),
					// the row (yy) needs to be floored before the rest of the calculation, because
					// we always need the exact row, or we basically get a wild salad of tiles
					tile = ((xx/tileSize)%size.x) + Math.floor((yy/tileSize)%size.y)*size.x;
					tile = Math.floor(tile);

					// skip this tile if it does not exist or is set to -1 (flag for don't draw)
					if(tiles[tile] !== undefined && tiles[tile] > -1){
						// no we draw directly from the sheet, rather than using the sheets SetIndex property
						// because this tilemap already has its own texture canvas
						ctx.drawImage(s.img, 
						tiles[tile] * tileSize%s.img.width,
						Math.floor(tiles[tile]*tileSize/(s.img.width))*tileSize,
						tileSize,tileSize,x-immidiateOffsetX,y-immidiateOffsetY,tileSize,tileSize);
					}

					this.trigger("drawtile",tile, x-immidiateOffsetX, y-immidiateOffsetY, tileSize, ctx, offsetX,offsetY);
				}
			}

			//draw child Entities of parent onto this texture
			ctx.translate(-this.attributes.scroll.x,-this.attributes.scroll.y);
			
			if(this.attributes.editable){
				ctx.strokeStyle= "rgba(0,255,255,.3)";
				ctx.strokeRect(.5,.5,size.x * tileSize - 1,size.y * tileSize - 1);
			}

			for(child = 0;child<this.attributes.entity.__childEntities.length;++child){
				
				this.attributes.entity.attributes.game.get("renderer").Render(
					this.attributes.entity.__childEntities[child]
					,ctx.canvas
					,true
				);
			}

			ctx.translate(offsetX,offsetY);
			ctx.restore();
		}
	});
});