define(['backbone','tools'],function($$,tools){
	return $$.Model.extend({
		defaults:{
			tilemap:null
			,game:null
			,colors:{
				hover:'rgba(125,255,255,.8)'
				,grid: 'rgba(255,255,255,.3)'
			}
			,brushes:{
				'default':null
			}
		}
		,active:false
		,visible:false
		,gui:null
		,hoverTile:null
		,brush:null
		,brushTile: 0
		,widget:null
		,initialize:function(attributes,options){
			var 
				self=this
				,toggleActive
				,toggleVisible
				,draw
				,keyboard
				,mouse
				,mousemove
			;

			if(!attributes ||!attributes.tilemap ||!attributes.game){
				tools.Warn("tried to initialize Tilemap Editor Class without valid tilemap attribute");
				return;
			}

			this.gui = attributes.game.get("editor");

			// game is not in the editor so nothing to do here...
			if(!this.gui){
				return;
			}

			this.InitWidget();

			// only show the editor functionality if the game update loop is stopped
			attributes.game.on("change:paused",toggleVisible = function(){
				self.visible = !!attributes.game.get("paused");
				self.trigger("visible",self.visible&&self.active);
			});
			toggleVisible();

			// only show the editor if the tilemap module is selected
			this.gui.on("change:selectedObject",toggleActive = function(){
				self.active = self.gui.get("selectedObject") === attributes.tilemap;
				self.trigger("visible",self.visible&&self.active);
			});
			toggleActive();

			mouse = attributes.game.GetMouse();
			keyboard = attributes.game.GetKeyboard();

			//draw outlines if active && visible
			attributes.tilemap.on("drawtile",draw = function(tile,x,y,size,ctx,oX,oY){
				var isRepeated=self.PositionIsRepeated(x,y,oX,oY), isHovered = self.TileIsHovered(tile);

				if(isRepeated)
					return;

				if(self.active && self.visible){
					ctx.lineCap = 'butt';
					ctx.strokeStyle = isHovered?self.attributes.colors.hover:self.attributes.colors.grid;
					ctx.strokeRect(x +.5,y+.5,size - 1,size - 1);

					if(isHovered && mouse.IsDown()){
						self.attributes.tilemap.attributes.tiles[tile] = keyboard.IsDown("ctrl")?-1:self.brushTile;
					}
				}

				
			});

			mouse.on("mousemove",mousemove = function(){
				self.hoverTile = self.GetTileAtPosition(this.GetPosition());
			});

			this.on("destroy",function(){
				mouse.off("mousemove",mousemove);

				attributes.tilemap.off("drawtile",draw);

				attributes.game.off("change:paused",toggleVisible);
				self.visible = false;
				
				self.gui.off("change:selectedObject",toggleActive);
				self.active = false;
				self.gui = null;

				self.widget.trigger("destroy");
				self.widget = null;
			});
		}
		,TileIsHovered:function(tile){
			
			return this.hoverTile === tile;
		}
		,GetTileAtPosition:function(rawpos){
			var tm = this.attributes.tilemap, pos = tm.attributes.entity.GlobalToLocal(rawpos);

			pos.x += tm.attributes.scroll.x;
			pos.y += tm.attributes.scroll.y;

			return Math.floor(pos.y / tm.attributes.sheet.tilesize) * tm.attributes.size.x + Math.floor(pos.x / tm.attributes.sheet.tilesize);
		}
		,PositionIsRepeated:function(x,y,offsetX,offsetY){
			var minX = offsetX, maxX = minX + this.attributes.tilemap.attributes.size.x * this.attributes.tilemap.attributes.sheet.tilesize
				,minY = offsetY, maxY = minY + this.attributes.tilemap.attributes.size.y * this.attributes.tilemap.attributes.sheet.tilesize;

			return false;//(x < minX || x >= maxX || y < minY || y >= maxY);
		}
		,InitWidget:function(){
			var self=this;
			require(['editor/widgets/tilemap'],function(Widget){
				self.widget = new Widget({tilemapeditor:self});
				var pane = self.gui.panes.find(function(pane){return pane.get("role") == "widget";});
				self.widget.set("index",10000);
				pane.widgets.add(self.widget);
			});
		}
	});
});