define(['glsl','backbone','underscore', 'tools','stats', 'scenes/scene', 'controldevices/mouse', 'controldevices/keyboard'],function(glsl,$$,_,tools,Stats,Scene,Mouse,Keyboard){
	var instances = window.GamesRegistry = {};

	return $$.Model.extend({
		defaults: {
			'name': 			'untitled game'
			,'version': 		'0.0.1'
			,'resolutionX': 	800
			,'resolutionY': 	450
			,'zoom': 			5
			,'scaleMode':		'fit'
			,'domElementId':	'game'
			,'rendererClass': 	'renderer'
			,'stats':			true
			,'scenes': 			null
			,'paused': 			false
			,'debug': 			false
			,'editor': 			false
		}
		,_running:true
		,_stopTime:0
		,_mouseInst:null
		,_keyboardInst:null
		,dom:null
		,initialize:function(attributes,options){
			this.dom = document.getElementById(this.get("domElementId"));
			if(!this.dom){
				this.dom = document.createElement("div");
				this.dom.id = this.get("domElementId");
				document.body.appendChild(this.dom);
			}
			instances[this.get("name")] = this;
			this.InitCanvas();
			var t=this;
			this.InitRenderer(function(){
				t.on("change:scene",t.OnSceneChange);
				t.InitScenes();
				t.Loop();
			});
		}
		,InitCanvas:function(){
			var canvas = document.createElement('canvas'), buffer = document.createElement('canvas'), changeWidth, changeHeight, self=this;
			canvas.style.background = 'black';
			this.dom.appendChild(canvas);
			this.set('canvasElement',canvas);
			this.set('buffer',buffer);
			this.on('change:resolutionX',changeWidth = function(target, value){
				canvas.width = canvas.style.width = value;
				buffer.width = canvas.width / self.attributes.zoom;
				self.dom.style.width = canvas.width;
			});
			this.on('change:resolutionY',changeHeight = function(target, value){
				canvas.height = canvas.style.height = value;
				buffer.height = canvas.height / self.attributes.zoom;
				self.dom.style.height = canvas.height;
			});
			
			changeWidth(null,this.get('resolutionX') || 800);
			changeHeight(null,this.get('resolutionY') || 450);
		}
		,InitScenes:function(){
			var s,i;
			if((s = this.get('scenes'))){
				for(i in s){
					if(s[i].isFirstScene){
						this.SetScene(i);
					}
				}
			}
		}
		,SetScene:function(scene, callback){
			if(!scene)
				return false;

			var self = this, scenedata, sc;
			if(_.isObject(scene) && scene instanceof Scene){
				this.set("scene",scene);
				return true;
			}else if(_.isObject(scene)){
				scenedata = scene;
			}else if(_.isString(scene) && this.attributes.scenes[scene]){
				scenedata = this.attributes.scenes[scene];
			}

			if(!scenedata){
				return false;
			}

			sc = scenedata.sceneClass || 'scenes/scene';

			require([sc],function(Scene){
				self.set("scene",new Scene(scenedata));
				callback&&callback(self.attributes.scene);
			});
			return true;
		}
		,GetScene:function(){
			return this.get("scene");
		}
		,OnSceneChange:function(target,scene){
			if(!target._changing)
				return;
			
			if(_.isObject(this._previousAttributes.scene)){
				this._previousAttributes.scene.set("parent",null);
			}

			if(_.isObject(scene)){
				scene.set("game",this);
			}
		}
		,InitRenderer:function(callback){
			var t=this;
			require([t.get('rendererClass')],function(renderer){
				t.set('renderer',new renderer);
				callback&&callback();
			});
		}
		,Loop:function(){
			var t=this,r=this.attributes.renderer,ctx=this.attributes.canvasElement.getContext("2d"),buffer=this.attributes.buffer, startTime = Date.now(), lastTime = this._stopTime || 0, tick, stats;
			if(!r||!buffer)
				return;

			if(t.get("stats")){
				if(t.dom.style.position != "absolute")
					t.dom.style.position = "relative";
				
				stats = new Stats();
				stats.domElement.style.position = "absolute";
				stats.domElement.style.left = "10px";
				stats.domElement.style.top = "10px";

				t.dom.appendChild(stats.domElement);

				t.on("change:paused",function(){
					stats.domElement.style.display = t.attributes.paused?"none":"block";
				});
				stats.domElement.style.display = t.attributes.paused?"none":"block";
			}
			
			(tick = function(){
				var delta, time=Date.now() - startTime + (t._stopTime || 0);
				
				if(!t._running){
					this._stopTime = time;
					return;
				}
				stats&&stats.begin();
				window.requestAnimationFrame(tick);
				
				if(!t.attributes.scene)
					return;
				
				delta = time - lastTime;

				lastTime = time;

				if(!t.attributes.paused)
					t.attributes.scene.trigger('update',time,delta);

				r.Render(t.attributes.scene,buffer);
				
				switch(t.attributes.scaleMode){
					case 'keepRatio':
						ctx.drawImage(buffer, 0, 0, buffer.width, buffer.height, 0, 0, ctx.canvas.width, ctx.canvas.width / (buffer.width * t.attributes.zoom) * (buffer.height * t.attributes.zoom));
					break;
					case 'fit':
						ctx.drawImage(buffer, 0, 0, buffer.width, buffer.height, 0, 0, ctx.canvas.width, ctx.canvas.height);
					break;
					case 'none':
					default:
						ctx.drawImage(buffer, 0, 0, buffer.width, buffer.height, 0, 0, buffer.width, buffer.height);
				}
				
				stats&&stats.end();
			})();
		}
		,GetMouse:function(){
			if(!this._mouseInst)
				this._mouseInst = new Mouse({element:this.get("canvasElement")});

			return this._mouseInst;			
		}
		,GetKeyboard:function(){
			if(!this._keyboardInst)
				this._keyboardInst = new Keyboard({element:this.get("canvasElement")});

			return this._keyboardInst;			
		}
	});
})