define(['backbone','underscore', 'entities/entity', 'tools', 'scenes/scene'],function($$,_,Entity,tools,Scene){
	return $$.Model.extend({
		defaults:{
			clear:true
			,clearColor:"rgb(0,0,0)"
		}
		,Render:function(scene,canvas,isSubTree){
			// return false if not setup correctly
			if(!_.isObject(scene) || !_.isObject(canvas))
				return false;
			var ctx = canvas.getContext("2d");

			if(this.attributes.clear && !isSubTree){
				if(this.attributes.clearColor !== null){
					ctx.fillStyle = this.attributes.clearColor;
					ctx.fillRect(0,0,ctx.canvas.width,ctx.canvas.height);
				}else{
					ctx.clearRect(0,0,ctx.canvas.width,ctx.canvas.height);	
				}
			}

			return (function RenderObject(obj){
				
				//translate & draw
				ctx.save();

				//console.log(obj.get("name"),(obj instanceof Scene));

				if(!(obj instanceof Scene) && obj.visible){

					//translate canvas to current objects position
					ctx.translate(obj.attributes.position.x, obj.attributes.position.y);

					//rotate the if the current object has rotation
					if(obj.attributes.rotation !== 0)
						ctx.rotate(obj.attributes.rotation);

					//scale if current object has scale
					if(obj.attributes.scale.x !== 1 || obj.attributes.scale.y !== 1)
						ctx.scale(obj.attributes.scale.x, obj.attributes.scale.y);

					//dispatch object drawEvent

					obj.trigger("draw",ctx);

					if(obj.texture && obj.texture.width && obj.texture.height){						
						ctx.drawImage(obj.texture,obj.attributes.origin.x,obj.attributes.origin.y);

						if(obj.attributes && obj.attributes.drawOutline){
							ctx.strokeStyle = "#ff0000";
							ctx.lineCap = 'butt';
							ctx.strokeRect(obj.attributes.origin.x + .5,obj.attributes.origin.y + .5,obj.texture.width -1,obj.texture.height -1);
						}
					}
				}

				// return true if there is nothing to do
				if(!obj.__childEntities || !obj.__childEntities.length||!obj.__drawChildEntities){
					ctx.restore();
					return true;
				}
					

				// render children recursively
				var i = 0, success = true;
				while(i < obj.__childEntities.length){
					if(!RenderObject(obj.__childEntities[i]))
						success = false;
					++i;
				}

				ctx.restore();
				return success;
			})(scene);
		}
	});
});