define(['backbone','underscore','tools','entities/entity','entities/container'],function($$,_,tools,Entity,Container){
	return Container.extend({
		meta:{
			className:"scenes/scene"
			,attributes:{
				'name':'String'
				,'entities':"entities/entity"
			}
		}
		,defaults:{
			'name': 'unnamed-scene'
			,'configFile':null
			,'game':null
			,'entities':null
		}
		,initialize:function(attributes, options){
			var i,ent;
			
			if(this.attributes.configFile){
				this.LoadConfig();
			}

			if(this.attributes.entities){
				for(i=0;i<this.attributes.entities.length;++i){
					ent = new Entity(this.attributes.entities[i]);
					if(ent.ready){
						ent.visible = true;
					}else{
						ent.visible = false;
						ent.on("ready",function(){
							this.visible = true;
						});
					}
					this.AddEntity(ent);
				}
			}

			//delegate events
			this.on("update",function(time,delta){
				var self=this;
				_.each(this.__childEntities,function(entity){entity.trigger("update",time,delta);});
			});

			this.on("change:game",function(){
				var self=this;
				_.each(this.__childEntities,function(entity){entity.set("game",self.attributes.game);});
			});
		}
		,LoadConfig:function(){
			var json = tools.FileGetJSON(this.get("configFile"));
			if(!_.isObject(json)){
				tools.Warn("Failed to Load Scene "+this.get("name"));
				return;
			}
			_.extend(this.attributes, json);
		}
		,toJSON:function(){
			var copy = _.clone(this.attributes);
			delete copy.game;
			delete copy.entities;
			
			if(this.__childEntities){
				copy.entities = [];
				_.each(this.__childEntities,function(ent){copy.entities.push(ent.toJSON())});
			}

			return copy;
		}
	});
});