define(['backbone','tools','underscore'],function($$,tools,_){
	// TexturePacker compatible sprite renderer
	return $$.Model.extend({
		defaults:{
			'image':null
			,'config':null
			,'tilesize':null
		}
		,texture:null
		,img:null
		,ready:false
		,index:null
		,config:null
		,currentFrameData:null
		,tileMode:false
		,initialize:function(attributes,options){
			if(attributes && attributes.image){
				this.InitSheet();
			}
		}
		,InitSheet:function(){
			var self=this, config = self.get("config");
			self.texture = document.createElement('canvas');
			
			if(config === null || config === undefined){
				if(this.attributes.tilesize){
					this.tileMode = true;
				}else{
					tools.Warn("corrupt configuration of Sprite" + this.get("image"));
				}
			}else if(_.isString(config)){
				this.config = tools.FileGetJSON(config);
			}else {
				this.config = config;
			}

			tools.LoadImage(this.get("image"), function(img){
				self.img = img;
				self.SetIndex(0);
				self.ready = true;
				self.trigger("ready");
			});
		}
		,SetIndex:function(index){
			if(this.index===index && index!==null){
				return;
			}
			
			if(!this.tileMode && !this.config.frames[index]){
				tools.Error("Sprite-Index "+index+" does not exists in spritesheet "+this.get("config"));
				return;
			}

			this.index = index;

			if(this.tileMode){
				this.DrawTile();
			}else{
				this.DrawSprite();	
			}
		}
		,DrawSprite:function(){
			var ctx = this.texture.getContext("2d"), f = this.currentFrameData = this.config.frames[this.index];
			ctx.canvas.width = f.frame.w;
			ctx.canvas.height = f.frame.h;
			ctx.clearRect(0,0,f.frame.w,f.frame.h);
			ctx.drawImage(this.img, f.frame.x, f.frame.y, f.frame.w, f.frame.h, 0, 0, f.frame.w, f.frame.h);
		}
		,DrawTile:function(){
			var ctx = this.texture.getContext("2d"),
			ts = this.get("tilesize") 
			xpos = this.index * ts%this.img.width,
			ypos = parseInt(this.index * ts / this.img.width) * ts;
			ctx.canvas.width = ctx.canvas.height = ts;
			ctx.clearRect(0,0,ts,ts);
			ctx.drawImage(this.img,xpos,ypos,ts,ts,0,0,ts,ts);
		}
	});
});